'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var reactDom = require('react-dom');
var earthstar = require('earthstar');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var React__namespace = /*#__PURE__*/_interopNamespace(React);

const PeerContext = React__namespace.createContext(new earthstar.Peer());
const ReplicaServersContext = React__namespace.createContext({ replicaServers: [], setReplicaServers: () => [] });
const KeypairContext = React__namespace.createContext({ keypair: null, setKeypair: () => { } });
const CurrentShareContext = React__namespace.createContext({
    currentShare: null,
    setCurrentShare: () => { },
});
const ShareSecretsContext = React__namespace.createContext({
    secrets: {},
    setSecrets: () => {
    }
});
const AddShareContext = React__namespace.createContext(async () => { });

function getAuthorShortName(address) {
    const parsedAuthor = earthstar.parseAuthorAddress(address);
    if (earthstar.isErr(parsedAuthor)) {
        return address;
    }
    return parsedAuthor.name;
}
const WORKSPACE_NAME_REGEX = /\+(.*)\./;
function getWorkspaceName(address) {
    const result = WORKSPACE_NAME_REGEX.exec(address);
    if (result) {
        return result[1];
    }
    return address;
}
function makeStorageKey(customKey, key) {
    if (!customKey) {
        return `earthstar:peer:${key}`;
    }
    return `earthstar:peer:${customKey}-${key}`;
}
function getLocalStorage(key) {
    const value = localStorage.getItem(key);
    if (!value) {
        return null;
    }
    try {
        return JSON.parse(value);
    }
    catch {
        return null;
    }
}

function usePeer() {
    const peer = React__namespace.useContext(PeerContext);
    const [trigger, setTrigger] = React__namespace.useState(true);
    const memoPeer = React__namespace.useMemo(() => peer, [trigger]);
    React__namespace.useEffect(() => {
        const unsub = peer.onReplicasChange(() => {
            setTrigger((prev) => !prev);
        });
        return () => {
            unsub();
        };
    }, [peer]);
    return memoPeer;
}
function useAddShare() {
    return React__namespace.useContext(AddShareContext);
}
function useShareSecret(address) {
    const { secrets, setSecrets } = React__namespace.useContext(ShareSecretsContext);
    const secret = secrets[address];
    const setSecret = React__namespace.useCallback((newSecret) => {
        setSecrets((prev) => ({
            ...prev,
            [secret]: newSecret
        }));
    }, []);
    return [secret, setSecret];
}
function useReplicaServers() {
    const { replicaServers, setReplicaServers } = React__namespace.useContext(ReplicaServersContext);
    return [replicaServers, setReplicaServers];
}
function useKeypair() {
    const { keypair, setKeypair } = React__namespace.useContext(KeypairContext);
    return [keypair, setKeypair];
}
function useCurrentShare() {
    const peer = usePeer();
    const { currentShare, setCurrentShare } = React__namespace.useContext(CurrentShareContext);
    React__namespace.useEffect(() => {
        if (currentShare && peer.hasShare(currentShare) === false) {
            console.warn(`Tried to set current workspace to ${currentShare}, which is not a known workspace.`);
            setCurrentShare(null);
        }
    }, [currentShare, setCurrentShare]);
    return [currentShare, setCurrentShare];
}
function useReplica(shareAddress) {
    const [currentShare] = useCurrentShare();
    const peer = usePeer();
    const address = shareAddress || currentShare;
    const replicaCache = React__namespace.useMemo(() => {
        const replica = address ? peer.getReplica(address) : undefined;
        if (!replica) {
            throw new Error("Tried to use useReplica with no share specified!");
        }
        return new earthstar.ReplicaCache(replica, 1000, (cb) => {
            reactDom.unstable_batchedUpdates(cb);
        });
    }, [address, peer]);
    const [, setTrigger] = React__namespace.useState(true);
    React__namespace.useLayoutEffect(() => {
        const unsub = replicaCache.onCacheUpdated(() => {
            setTrigger((prev) => !prev);
        });
        return () => {
            unsub();
            replicaCache.close();
        };
    }, [replicaCache]);
    return replicaCache;
    // Keeping the below around for React 18.
    /*
      const replicaCache = React.useMemo(() => {
        const replica = address ? peer.getReplica(address) : undefined;
           if (!replica) {
          throw new Error("Tried to use useReplica with no share specified!");
        }
           return new ReplicaCache(replica, 1000);
         [address, peer])
         const [currentVersion, setVersion] = React.useState(replicaCache.version);
         const memoReplica = React.useMemo(() => {
        return replicaCache;
         [currentVersion, replicaCache]);
         React.useEffect(() => {
        replicaCache.onCacheUpdated(() => {
          setVersion(replicaCache.version);
  
        ;
           nst subscribe = React.useCallback((onStoreChange: () => void) => {
          turn memoReplica.onCacheUpdated(() => {
            StoreChange();
          ;
         [memoReplica, replicaCache]);
           nst getSnapshot = React.useCallback(() => memoReplica, []);
           turn useSyncExternalStoreWithSelector(
          bscribe,
          tSnapshot,
          tSnapshot,
          ache) => cache,
          acheA, cacheB) => cacheA.version === cacheB.version,
        */
}
function useSync(target) {
    const peer = usePeer();
    const syncer = peer.sync(target);
    const [status, setStatus] = React__namespace.useState(syncer.getStatus);
    React__namespace.useEffect(() => {
        return syncer.onStatusChange((status) => {
            setStatus(status);
        });
    }, [syncer]);
    return [syncer, status];
}
function useInvitation(invitationCode) {
    const addShare = React__namespace.useContext(AddShareContext);
    const [, setPubs] = useReplicaServers();
    try {
        const url = new URL(invitationCode);
        const isEarthstarURL = url.protocol === "earthstar:";
        if (!isEarthstarURL) {
            return new earthstar.EarthstarError("Invitation not a valid Earthstar URL");
        }
        const version = url.searchParams.get("v");
        if (version !== "1") {
            return new earthstar.EarthstarError("Unrecognised Earthstar invitation format version");
        }
        const workspace = url.searchParams.get("workspace");
        if (workspace === null) {
            return new earthstar.EarthstarError("No workspace found in Earthstar invitation URL");
        }
        const plussedWorkspace = workspace.replace(" ", "+");
        const shareIsValid = earthstar.checkShareIsValid(plussedWorkspace);
        if (earthstar.isErr(shareIsValid)) {
            return shareIsValid;
        }
        const pubs = url.searchParams.getAll("pub");
        try {
            pubs.forEach((pubUrl) => new URL(pubUrl));
        }
        catch {
            return new earthstar.EarthstarError("Malformed Pub URL found");
        }
        const redeem = (excludedPubs = []) => {
            addShare(plussedWorkspace);
            const nextPubs = Array.from(new Set([
                ...pubs.filter((pubUrl) => !excludedPubs.includes(pubUrl)),
            ]));
            setPubs((prevPubs) => Array.from(new Set([
                ...prevPubs,
                ...nextPubs,
            ])));
        };
        return { redeem, workspace: plussedWorkspace, pubs };
    }
    catch {
        return new earthstar.EarthstarError("Not a valid Earthstar URL");
    }
}
function useMakeInvitation(includedPubs = [], shareAddress) {
    const [pubs] = useReplicaServers();
    const [currentShare] = useCurrentShare();
    const address = shareAddress || currentShare;
    const pubsToUse = pubs.filter((pubUrl) => includedPubs.includes(pubUrl));
    const pubsString = pubsToUse.map((pubUrl) => `&pub=${pubUrl}`).join("");
    if (!address) {
        return "Couldn't create invitation code!";
    }
    return `earthstar:///?workspace=${address}${pubsString}&v=1`;
}
function useLocalStorageEarthstarSettings(storageKey) {
    const lsAuthorKey = makeStorageKey(storageKey, "author");
    const lsServersKey = makeStorageKey(storageKey, "replica-servers");
    const lsSharesKey = makeStorageKey(storageKey, "shares");
    const lsCurrentShareKey = makeStorageKey(storageKey, "current-share");
    const lsShareSecretsKey = makeStorageKey(storageKey, "share-secrets");
    const allKeys = React__namespace.useMemo(() => [
        lsAuthorKey,
        lsServersKey,
        lsSharesKey,
        lsCurrentShareKey,
        lsShareSecretsKey,
    ], [
        lsAuthorKey,
        lsServersKey,
        lsSharesKey,
        lsCurrentShareKey,
        lsShareSecretsKey,
    ]);
    // load the initial state from localStorage
    const initShares = getLocalStorage(lsSharesKey);
    const initReplicaServers = getLocalStorage(lsServersKey);
    const initIdentity = getLocalStorage(lsAuthorKey);
    const initCurrentShare = getLocalStorage(lsCurrentShareKey);
    const initShareSecrets = getLocalStorage(lsShareSecretsKey);
    const [, setTrigger] = React__namespace.useState(true);
    const onStorage = React__namespace.useCallback((event) => {
        console.log(event.key);
        if (event.key && allKeys.includes(event.key)) {
            setTrigger((prev) => !prev);
        }
    }, [allKeys]);
    React__namespace.useEffect(() => {
        window.addEventListener("storage", onStorage);
        return () => {
            window.removeEventListener("storage", onStorage);
        };
    }, [onStorage]);
    return {
        initShares: initShares || [],
        initShareSecrets: initShareSecrets || {},
        initReplicaServers: initReplicaServers || [],
        initIdentity: initIdentity || null,
        initCurrentShare: initCurrentShare || null,
    };
}

function Peer({ initShares = [], initShareSecrets = {}, initReplicaServers = [], initIdentity = null, initCurrentShare = null, onCreateShare, children, }) {
    const peer = React__namespace.useMemo(() => {
        const p = new earthstar.Peer();
        console.log(initShares, initShareSecrets);
        initShares.forEach((address) => {
            p.addReplica(onCreateShare(address, initShareSecrets[address]));
        });
        return p;
    }, []);
    const [secrets, setSecrets] = React__namespace.useState(initShareSecrets);
    const [replicaServers, setReplicaServers] = React__namespace.useState(initReplicaServers);
    const [keypair, setKeypair] = React__namespace.useState(initIdentity);
    const [currentShare, setCurrentShare] = React__namespace.useState(initCurrentShare);
    const addShare = React__namespace.useCallback(async (shareAddress, secret) => {
        try {
            const storage = onCreateShare(shareAddress);
            await peer.addReplica(storage);
            if (secret) {
                setSecrets((prev) => ({ ...prev, [shareAddress]: secret }));
            }
        }
        catch (err) {
            console.error(err);
            return;
        }
    }, []);
    React__namespace.useEffect(() => {
        peer.onReplicasChange((replicas) => {
            const addresses = new Set(replicas.keys());
            setSecrets((prev) => {
                const next = {};
                for (const key in prev) {
                    if (addresses.has(key)) {
                        next[key] = prev[key];
                    }
                }
                return prev;
            });
        });
    });
    return (React__namespace.createElement(PeerContext.Provider, { value: peer },
        React__namespace.createElement(ShareSecretsContext.Provider, { value: { secrets, setSecrets } },
            React__namespace.createElement(ReplicaServersContext.Provider, { value: { replicaServers, setReplicaServers } },
                React__namespace.createElement(KeypairContext.Provider, { value: { keypair, setKeypair } },
                    React__namespace.createElement(CurrentShareContext.Provider, { value: { currentShare, setCurrentShare } },
                        React__namespace.createElement(AddShareContext.Provider, { value: addShare }, children)))))));
}

function IdentityLabel({ address, ...props }) {
    return (React__namespace.createElement("span", { ...props, title: address }, `@${getAuthorShortName(address)}`));
}

function CurrentIdentityLabel(props) {
    const [currentAuthor] = useKeypair();
    return currentAuthor
        ? React__namespace.createElement(IdentityLabel, { ...props, address: currentAuthor.address })
        : React__namespace.createElement(React__namespace.Fragment, null, "Not signed in");
}

function LocalStorageSettingsWriter({ storageKey, }) {
    const lsAuthorKey = makeStorageKey(storageKey, "author");
    const lsServersKey = makeStorageKey(storageKey, "replica-servers");
    const lsSharesKey = makeStorageKey(storageKey, "shares");
    const lsCurrentShareKey = makeStorageKey(storageKey, "current-share");
    const lsShareSecretsKey = makeStorageKey(storageKey, "share-secrets");
    const peer = usePeer();
    const secrets = React__namespace.useContext(ShareSecretsContext);
    const [pubs] = useReplicaServers();
    const [currentIdentity] = useKeypair();
    const [currentShare] = useCurrentShare();
    React__namespace.useEffect(() => {
        return peer.onReplicasChange(() => {
            localStorage.setItem(lsSharesKey, JSON.stringify(peer.shares()));
        });
    }, [peer, lsSharesKey]);
    React__namespace.useEffect(() => {
        localStorage.setItem(lsServersKey, JSON.stringify(pubs));
    }, [pubs, lsServersKey]);
    React__namespace.useEffect(() => {
        localStorage.setItem(lsAuthorKey, JSON.stringify(currentIdentity));
    }, [currentIdentity, lsAuthorKey]);
    React__namespace.useEffect(() => {
        localStorage.setItem(lsShareSecretsKey, JSON.stringify(secrets));
    }, [secrets, lsShareSecretsKey]);
    React__namespace.useEffect(() => {
        console.log(currentShare);
        localStorage.setItem(lsCurrentShareKey, JSON.stringify(currentShare));
    }, [currentShare, lsCurrentShareKey]);
    return null;
}

/*

WorkspaceLabel

Given a workspace address, returns an element which:
- Displays the workspace name without the hash (so as not to accidentally expose the address to others)
- Ensures the workspace name is displayed with a preceding + sigil (so as to signify it being a workspace)
- Adds a title attribute with the full workspace address (so the user can check the full address if needed)

*/
function ShareLabel({ address, ...props }) {
    return (React__namespace.createElement("span", { ...props, title: address }, `+${getWorkspaceName(address)}`));
}

exports.CurrentIdentityLabel = CurrentIdentityLabel;
exports.IdentityLabel = IdentityLabel;
exports.LocalStorageSettingsWriter = LocalStorageSettingsWriter;
exports.Peer = Peer;
exports.ShareLabel = ShareLabel;
exports.useAddShare = useAddShare;
exports.useCurrentShare = useCurrentShare;
exports.useInvitation = useInvitation;
exports.useKeypair = useKeypair;
exports.useLocalStorageEarthstarSettings = useLocalStorageEarthstarSettings;
exports.useMakeInvitation = useMakeInvitation;
exports.usePeer = usePeer;
exports.useReplica = useReplica;
exports.useReplicaServers = useReplicaServers;
exports.useShareSecret = useShareSecret;
exports.useSync = useSync;
